"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.normalizeConfig = exports.validateConfig = void 0;
const visitor_plugin_common_1 = require("@graphql-codegen/visitor-plugin-common");
function validateConfig(rawConfig) {
    if (typeof rawConfig !== 'object' || rawConfig === null) {
        throw new Error('`options` must be an object');
    }
    if (!('typesFile' in rawConfig)) {
        throw new Error('`option.typesFile` is required');
    }
    if (typeof rawConfig['typesFile'] !== 'string') {
        throw new Error('`options.typesFile` must be a string');
    }
    if ('skipTypename' in rawConfig && typeof rawConfig['skipTypename'] !== 'boolean') {
        throw new Error('`options.skipTypename` must be a boolean');
    }
    if ('skipIsAbstractType' in rawConfig && typeof rawConfig['skipIsAbstractType'] !== 'boolean') {
        throw new Error('`options.skipIsAbstractType` must be a boolean');
    }
    if ('nonOptionalDefaultFields' in rawConfig && typeof rawConfig['nonOptionalDefaultFields'] !== 'boolean') {
        throw new Error('`options.nonOptionalDefaultFields` must be a boolean');
    }
    if ('typesPrefix' in rawConfig && typeof rawConfig['typesPrefix'] !== 'string') {
        throw new Error('`options.typesPrefix` must be a string');
    }
    if ('typesSuffix' in rawConfig && typeof rawConfig['typesSuffix'] !== 'string') {
        throw new Error('`options.typesSuffix` must be a string');
    }
}
exports.validateConfig = validateConfig;
function normalizeConfig(rawConfig) {
    return {
        typesFile: rawConfig.typesFile,
        skipTypename: rawConfig.skipTypename ?? false,
        skipIsAbstractType: rawConfig.skipIsAbstractType ?? true,
        nonOptionalDefaultFields: rawConfig.nonOptionalDefaultFields ?? false,
        typesPrefix: rawConfig.typesPrefix ?? '',
        typesSuffix: rawConfig.typesSuffix ?? '',
        convert: rawConfig.namingConvention
            ? (0, visitor_plugin_common_1.convertFactory)({ namingConvention: rawConfig.namingConvention })
            : (0, visitor_plugin_common_1.convertFactory)({}),
    };
}
exports.normalizeConfig = normalizeConfig;
//# sourceMappingURL=config.js.map