"use strict";
// MEMO: The tests for this module are covered by `e2e/*.e2e.ts`.
Object.defineProperty(exports, "__esModule", { value: true });
exports.defineTypeFactoryInternal = void 0;
const field_resolver_js_1 = require("./field-resolver.js");
const sequence_js_1 = require("./sequence.js");
function defineTypeFactoryInternal(typeFieldNames, { defaultFields: defaultFieldsResolver, traits, }) {
    const seqKey = {};
    const getSeq = () => (0, sequence_js_1.getSequenceCounter)(seqKey);
    return {
        async build(inputFieldsResolver) {
            const seq = getSeq();
            return (0, field_resolver_js_1.resolveFields)(typeFieldNames, seq, defaultFieldsResolver, inputFieldsResolver ?? {});
        },
        async buildList(count, inputFieldsResolver) {
            const array = [];
            for (let i = 0; i < count; i++) {
                if (inputFieldsResolver) {
                    // eslint-disable-next-line no-await-in-loop, @typescript-eslint/no-explicit-any
                    array.push((await this.build(inputFieldsResolver)));
                }
                else {
                    // eslint-disable-next-line no-await-in-loop, @typescript-eslint/no-explicit-any
                    array.push((await this.build()));
                }
            }
            return array;
        },
        use(traitName) {
            const trait = traits?.[traitName];
            if (!trait)
                throw new Error(`Trait ("${String(traitName)}") is not defined.`);
            // @ts-expect-error -- Use @ts-expect-error as it is impossible to match types.
            return defineTypeFactoryInternal(typeFieldNames, {
                defaultFields: { ...defaultFieldsResolver, ...trait.defaultFields },
                traits,
            });
        },
        resetSequence() {
            (0, sequence_js_1.resetSequence)(seqKey);
        },
    };
}
exports.defineTypeFactoryInternal = defineTypeFactoryInternal;
//# sourceMappingURL=factory.js.map