"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getTypeInfos = void 0;
const visitor_plugin_common_1 = require("@graphql-codegen/visitor-plugin-common");
const graphql_1 = require("graphql");
// The fork of https://github.com/dotansimha/graphql-code-generator/blob/e1dc75f3c598bf7f83138ca533619716fc73f823/packages/plugins/typescript/resolvers/src/visitor.ts#L85-L91
function clearOptional(str) {
    if (str.startsWith('Maybe')) {
        return str.replace(/Maybe<(.*?)>$/u, '$1');
    }
    return str;
}
// The fork of https://github.com/dotansimha/graphql-code-generator/blob/ba84a3a2758d94dac27fcfbb1bafdf3ed7c32929/packages/plugins/other/visitor-plugin-common/src/base-visitor.ts#L422
function convertName(node, config) {
    let convertedName = '';
    convertedName += config.typesPrefix;
    convertedName += config.convert(node);
    convertedName += config.typesSuffix;
    return convertedName;
}
function isTypeBasedOnUserDefinedType(node, userDefinedTypeNames) {
    if (node.kind === graphql_1.Kind.NON_NULL_TYPE) {
        return isTypeBasedOnUserDefinedType(node.type, userDefinedTypeNames);
    }
    else if (node.kind === graphql_1.Kind.LIST_TYPE) {
        return isTypeBasedOnUserDefinedType(node.type, userDefinedTypeNames);
    }
    else {
        return userDefinedTypeNames.includes(node.name.value);
    }
}
function parseTypeNode(node, config) {
    if (node.kind === graphql_1.Kind.NON_NULL_TYPE) {
        return clearOptional(parseTypeNode(node.type, config));
    }
    else if (node.kind === graphql_1.Kind.LIST_TYPE) {
        return `Maybe<${parseTypeNode(node.type, config)}[]>`;
    }
    else {
        return `Maybe<Optional${convertName(node.name.value, config)}>`;
    }
}
function parseFieldOrInputValueDefinition(node, convertedTypeName, config, userDefinedTypeNames) {
    const comment = node.description ? (0, visitor_plugin_common_1.transformComment)(node.description) : undefined;
    if (isTypeBasedOnUserDefinedType(node.type, userDefinedTypeNames)) {
        return { typeString: `${parseTypeNode(node.type, config)} | undefined`, comment };
    }
    else {
        return { typeString: `${convertedTypeName}['${node.name.value}'] | undefined`, comment };
    }
}
function parseObjectTypeOrInputObjectTypeDefinition(node, config, userDefinedTypeNames, getAbstractTypeNames) {
    const originalTypeName = node.name.value;
    const convertedTypeName = convertName(originalTypeName, config);
    const comment = node.description ? (0, visitor_plugin_common_1.transformComment)(node.description) : undefined;
    const abstractTypeNames = node.kind === graphql_1.Kind.OBJECT_TYPE_DEFINITION ? getAbstractTypeNames(node) : [];
    return {
        name: convertedTypeName,
        fields: [
            ...(!config.skipTypename ? [{ name: '__typename', typeString: `'${originalTypeName}'` }] : []),
            ...(!config.skipIsAbstractType
                ? abstractTypeNames.map((name) => ({ name: `__is${name}`, typeString: `'${originalTypeName}'` }))
                : []),
            ...(node.fields ?? []).map((field) => ({
                name: field.name.value,
                ...parseFieldOrInputValueDefinition(field, convertedTypeName, config, userDefinedTypeNames),
            })),
        ],
        comment,
    };
}
function getTypeInfos(config, schema) {
    const types = Object.values(schema.getTypeMap());
    const objectTypeOrInputObjectTypeDefinitions = types
        .map((type) => type.astNode)
        .filter((node) => {
        if (!node)
            return false;
        return node.kind === graphql_1.Kind.OBJECT_TYPE_DEFINITION || node.kind === graphql_1.Kind.INPUT_OBJECT_TYPE_DEFINITION;
    });
    const unionTypeDefinitions = types
        .map((type) => type.astNode)
        .filter((node) => {
        if (!node)
            return false;
        return node.kind === graphql_1.Kind.UNION_TYPE_DEFINITION;
    });
    function getAbstractTypeNames(type) {
        const interfaceNames = (type.interfaces ?? []).map((i) => i.name.value);
        const unionNames = unionTypeDefinitions
            .filter((union) => (union.types ?? []).some((member) => member.name.value === type.name.value))
            .map((union) => union.name.value);
        return [...interfaceNames, ...unionNames];
    }
    const userDefinedTypeNames = objectTypeOrInputObjectTypeDefinitions.map((type) => type.name.value);
    const typeInfos = objectTypeOrInputObjectTypeDefinitions.map((node) => parseObjectTypeOrInputObjectTypeDefinition(node, config, userDefinedTypeNames, getAbstractTypeNames));
    return typeInfos;
}
exports.getTypeInfos = getTypeInfos;
//# sourceMappingURL=schema-scanner.js.map